unit UAppleRead1;
(*

 This program converts a standard Apple Pascal 280 block disk image 
(.DSK) to a .VOL file - later versions of the p-system can read the disk.

Once converted, your destination file (if named .VOL) can be mounted 
by the DOS (and Unix) hosted p-system

Laurie Boshell    laurieboshell@gmail.com

*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Md5DeftEdit, ComCtrls;

type
  TForm1 = class(TForm)
    DeftEdit1: TDeftEdit;
    Label1: TLabel;
    Button1: TButton;
    Memo1: TMemo;
    UpDown1: TUpDown;
    Label4: TLabel;
    Button2: TButton;
    Label5: TLabel;
    Button3: TButton;
    OpenDialog1: TOpenDialog;
    Label2: TLabel;
    DeftEdit2: TDeftEdit;
    Memo2: TMemo;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure UpDown1Click(Sender: TObject; Button: TUDBtnType);
    procedure Button3Click(Sender: TObject);
    procedure DeftEdit2Exit(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

Type
  TSectorData = packed array[0..255] of byte;
  TInfile = file of TSectorData;
  TOutfile= file of TSectorData;
  TBlock = Array[0..511] of byte;
  TSectors = record
             LoSector : TSectorData;  //first 256 byte sector of block
             HiSector : TSectorData;  //second 256 byte sector
             end;
  TBlockUnion = record case integer of
                1: (Buf : TBlock);
                2: (Sectors: TSectors);
                end;

var
  SectorTable : array[1..26] of integer;
  Infile      : TInfile;
  Outfile     : TOutfile;
  track       : 0..34;
  sector      : 0..15;
  BlockNum    : 0..279;   //Apple has 288 blocks
  mySector    : TSectorData;
  //Declare a Block as TBlockUnion
  //Access a block as Block.buf or Block.Sectors.LoSector

Procedure initSectorTable21;
Begin    //for 26 sector 8" CP/M floppy
  {2:1}
  sectortable[ 1] := 1;
  sectortable[ 2] := 3;
  sectortable[ 3] := 5;
  sectortable[ 4] := 7;
  sectortable[ 5] := 9;
  sectortable[ 6] :=11;
  sectortable[ 7] :=13;
  sectortable[ 8] :=15;
  sectortable[ 9] :=17;
  sectortable[10] :=19;
  sectortable[11] :=21;
  sectortable[12] :=23;
  sectortable[13] :=25;
  sectortable[14] := 2;
  sectortable[15] := 4;
  sectortable[16] := 6;
  sectortable[17] := 8;
  sectortable[18] :=10;
  sectortable[19] :=12;
  sectortable[20] :=14;
  sectortable[21] :=16;
  sectortable[22] :=18;
  sectortable[23] :=20;
  sectortable[24] :=22;
  sectortable[25] :=24;
  sectortable[26] :=26;
end;


Function SectorTranslate(t,s:integer):integer;
//apply 2:1 sector interleave
//then add skew offset of 6 based on track #
var
  SkewOffset:integer;
Begin
  s:=SectorTable[s];
  SkewOffset := ((t-1) mod 13) * 6;
  SkewOffset := SkewOffset mod 26;
  s:=s+SkewOffset;
  While s > 26 do begin
    s:=s-26;
    MessageBeep($FFFF{-1});
    end;
  SectorTranslate := s;
end;


Procedure ConvertBlock2TS(B:integer; VAR T,S1,S2:integer);   //Convert block to track and sectors
//For Apple II .DSK image in Apple Pascal
Begin
  T:=B DIV 8;
  Case B MOD 8 of {0..7 blocks per track}
    0 : begin S1:= 0; S2:=14; end;
    1 : begin S1:=13; S2:=12; end;
    2 : begin S1:=11; S2:=10; end;
    3 : begin S1:= 9; S2:= 8; end;
    4 : begin S1:= 7; S2:= 6; end;
    5 : begin S1:= 5; S2:= 4; end;
    6 : begin S1:= 3; S2:= 2; end;
    7 : begin S1:= 1; S2:=15; end;
    end;
end;


(*
Procedure ConvertBlock2TS(B:integer; VAR T,S1,S2:integer);   //Convert block to track and sectors
Begin
  T:=B DIV 8;
  S1:=(B MOD 8)* 2 ;
  S2 := S1+1;
end;
*)



Procedure Read2Sectors(T,S1,S2:integer; Var Block:TBlockunion);
var rec : Integer;
Begin
  rec := T*16 + S1;
  Form1.Label4.Caption := 'T = '+IntToStr(t)+' s='+IntToStr(s1)+' rec= '+IntToStr(rec);
  Seek(InFile,rec);
  Read(Infile,mySector);
  Block.Sectors.LoSector :=MySector;
  rec := T*16 + S2;
  Seek(InFile,rec);
  Read(Infile,mySector);
  Block.Sectors.HiSector :=MySector;
end;

Procedure ReadBlock(B:integer; Var Buf:TBlock);
var T,S1,S2: integer;
    Block:TBlockUnion;
Begin
  ConvertBlock2TS(B,T,S1,S2);   //Convert block to track and sectors
  Form1.Memo1.Lines.add('B='+IntToStr(B)+' T='+IntToStr(T)+' S1='+IntToStr(S1)
                        +' S2='+IntToStr(S2));
  Read2Sectors(T,S1,S2,Block);
  Buf := Block.Buf;
end;

Procedure WriteSector(T,S:integer);
var rec : Integer;
Begin
  rec := T *26 + s;
  Seek(outFile,rec);
  Write(Outfile,mySector);
end;

Procedure WriteBlock(B:integer; Buf:TBlock);
var rec : Integer;
    Block:TBlockUnion;
Begin
  Block.Buf := Buf;
  rec := B *2;    //first 256 byte sector of block
  Seek(outFile,rec);
  Write(Outfile,Block.Sectors.Losector);
  rec := rec+1;   //second 256 byte sector of block
  Seek(outFile,rec);
  Write(Outfile,Block.Sectors.Hisector);
end;

procedure TForm1.Button1Click(Sender: TObject);
var b, t,s:integer;  Buf:TBlock;
begin
  if DeftEdit1.text = DeftEdit2.text then begin
    ShowMessage('The source and destination file names are the same!. Change destination name');
    exit
    end;
  Button2.Enabled := false;
  UpDown1.Enabled := false;

  initsectortable21;
  AssignFile(infile,DeftEdit1.text{'c:\iisource\u1301bas.vol'});
  reset(infile);
  AssignFile(outfile,DeftEdit2.text); //'c:\psystem\temp.vol'); {iisource\outfile.vol');}
  rewrite(outfile);

  for b := 0 to 279 do begin  //Apple II disk has 280 blocks
    ReadBlock(b,Buf);
    WriteBlock(b,Buf);
    end;
  closeFile(outfile);
  closeFile(infile);
  //Now reset globals
  track:=0;   //Ignore track 0
  Sector := 0;
  ShowMessage('Disc IMAGE converted, saved in '+DeftEdit2.text);
  Button2.Enabled := true;
end;


Procedure DisplayBlock(Buf:TBlock);
var s:string; offset,i:integer; m:byte;
Begin
//  Form1.Memo1.Clear;
  Form1.Memo1.Lines.add('B='+IntToStr(BlockNum));
  for offset := 0 to 7 do begin
    s:='';
    //some calculations to show 1 lines of text per 128 byte sectors
    for i:= 1 to 64 do begin
      m:= Buf[offset*64 +i -1];
      If m > 127 then m := m - 128;
      If (m> 31) and (m<127) then
        s:=s+chr(m)
        Else s:=s+' ';
      end;
      Form1.Memo1.Lines.Add(s);
    end;  //offset
end;

var infileopened : boolean;    //Remember to close the file on the way out
procedure TForm1.Button2Click(Sender: TObject);
Var Buf:TBlock;
begin
  AssignFile(infile,DeftEdit1.text{'c:\iisource\u1301bas.vol'});
  reset(infile);
  InFileOpened := true;
  ReadBlock(BlockNum, Buf);
  DisplayBlock(Buf);
  UpDown1.Enabled := true;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  If infileOpened then CloseFile(Infile);
end;

procedure TForm1.UpDown1Click(Sender: TObject; Button: TUDBtnType);
Var Buf:TBlock;
begin
  if button = btPrev then begin  //Prev sectors
    if BlockNum = 0 then // do nothing
    else Dec(BlockNum);
    end
  else begin                     //Next sectors
    if BlockNum=279 then // do nothing
    else Inc(BlockNum);
    end;
  Label5.Caption := 'BLOCK = '+IntToStr(BlockNum);
  readBlock(BlockNum,Buf);
  DisplayBlock(Buf);
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  If OpenDialog1.execute then begin
    DeftEdit1.Text := OpenDialog1.FileName
    end;
end;

procedure TForm1.DeftEdit2Exit(Sender: TObject);
begin
  if DeftEdit2.Text = DeftEdit1.text then begin
    ShowMessage('The destination is the same as the source file');
    DeftEdit2.text:='';
    DeftEdit2.SetFocus;
    end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 infileOpened := false;
 BlockNum :=0;
end;

end.
